/////////////////////////////////////////////////////////////////////////////
// Name:        wxMySQL.h
// Purpose:     wxMyQSL (v0.1.2)
// Author:      Volodymir (T-Rex) Tryapichko
// Modified by:
// Created:     Feb-19-2005
// RCS-ID:      $Id:
// Copyright:   (c) Volodymir (T-Rex) Tryapichko
// Licence:     wxWidgets license
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_MYSQL_H
#define _WX_MYSQL_H

#include <wx/wx.h>
#include <wx/list.h>
#ifdef __WXMSW__
	#include <windows.h>
#endif
#include <mysql.h>

enum wxMySQLFieldType 
{	
	wxMYSQL_TYPE_DECIMAL, 
	wxMYSQL_TYPE_TINY,
	wxMYSQL_TYPE_SHORT,  
	wxMYSQL_TYPE_LONG,
	wxMYSQL_TYPE_FLOAT,  
	wxMYSQL_TYPE_DOUBLE,
	wxMYSQL_TYPE_NULL,   
	wxMYSQL_TYPE_TIMESTAMP,
	wxMYSQL_TYPE_LONGLONG,
	wxMYSQL_TYPE_INT24,
	wxMYSQL_TYPE_DATE,   
	wxMYSQL_TYPE_TIME,
	wxMYSQL_TYPE_DATETIME, 
	wxMYSQL_TYPE_YEAR,
	wxMYSQL_TYPE_NEWDATE,
	wxMYSQL_TYPE_ENUM=247,
	wxMYSQL_TYPE_SET=248,
	wxMYSQL_TYPE_TINY_BLOB=249,
	wxMYSQL_TYPE_MEDIUM_BLOB=250,
	wxMYSQL_TYPE_LONG_BLOB=251,
	wxMYSQL_TYPE_BLOB=252,
	wxMYSQL_TYPE_VAR_STRING=253,
	wxMYSQL_TYPE_STRING=254,
	wxMYSQL_TYPE_GEOMETRY=255
};

wxString wxMySQLGetFieldTypeName(wxMySQLFieldType type);

class wxMySQLField
{
public:
	wxString m_Name;
	wxString m_NameOrg;
	wxString m_TableName;
	wxString m_TableNameOrg;
	wxString m_DBName;
	wxString m_DBCatalog;
	wxString m_DefaultValue;
	unsigned long m_Length;
	unsigned long m_MaxLength;
	unsigned int m_Decimals;
	unsigned int m_Flags;
	wxMySQLFieldType m_FieldType;
	bool IsNotNULL();
	bool IsPrimaryKeyField();
	bool IsUniqueKeyField();
	bool IsNonUniqueKeyField();
	bool IsUnSigned();
	bool IsZeroFill();
	bool IsBinary();
	bool IsAutoIncrement();
	bool IsEnum();
	bool IsSet();
	bool IsBLOB();
	bool IsTimeStamp();	
};

WX_DECLARE_OBJARRAY(wxMySQLField, wxMySQLFieldArray);

class wxMySQLResult
{		
	MYSQL_RES * m_Res;	
public:			
	wxMySQLResult();
	~wxMySQLResult();
	void SetHandle(MYSQL_RES * res);
	void Clear();	
	unsigned int GetFieldCount();
	unsigned long GetRowCount();
	bool Fetch(wxStringList & list);
	bool Fetch(wxArrayString & array);
	bool GetFieldInfo(wxMySQLField * field, unsigned int fieldnum);
	bool GetFieldsInfo(wxMySQLFieldArray & array);
	void SeekRow(unsigned long index);
};

class wxMySQLRecordset
{
	MYSQL_STMT * m_Stmt;	
	MYSQL_BIND * m_ColDefs;
	void SetFieldCount(unsigned int value);
public:
	wxMySQLRecordset();
	unsigned int GetFieldCount();
	void SetHandle(MYSQL_STMT * stmt);	
	void SetFieldType(unsigned int index, wxMySQLFieldType type);
	void SetFieldBuffer(unsigned int index, void * buffer);
	void SetFieldBufferLength(unsigned int index, unsigned long * length);
	bool BindFields();
	bool Execute();
	bool Close();
	bool IsOk();
	bool Prepare(wxString query);
	wxString GetLastErrorMessage();	
};

class wxMySQL
{
	wxString m_Hostname;
	int m_Port;
	wxString m_Username;
	wxString m_Password;
	wxString m_DatabaseName;
	MYSQL * m_pDB;
	wxString m_LastErrorMessage;
public:
	wxMySQL();
	wxMySQL(wxString hostname, int port, wxString username, wxString password, wxString dbname = "");
	~wxMySQL();
	bool Connect(wxString hostname, int port, wxString username, wxString password, wxString dbname = "");
	bool ChangeUser(wxString username, wxString password, wxString dbname = "");
	bool IsConnected();
	bool EnumDatabases(wxStringList & list);
	bool EnumDatabases(wxArrayString & array);
	bool EnumTables(wxStringList & list, wxString wildchar = "");
	bool EnumTables(wxArrayString & array, wxString wildchar = "");
	bool CreateDatabase(wxString dbname);
	bool OpenDatabase(wxString dbname);
	bool DropDatabase(wxString dbname);
	bool Execute(wxString query, wxMySQLResult * result = NULL);
	bool CreateRecordset(wxString query, wxMySQLRecordset * result = NULL);
	unsigned long GetAffectedRows();
	unsigned int GetTableCount();		
	wxString GetLastErrorMessage();
	void Disconnect();	
};

#if wxMYSQL_BACKWARD_COMPATIBILITY
#define DoQuery Execute
#endif

#endif